import pandas as pd
import dataiku
import json

def interface_to_dataset(message, slots_dataset, cache_dataset):
    # Converting message format to dataframe:
    message_data = json.loads(message)
    datalist = []
    for i in message_data:
        d = {'dock': i["name"], 'slots': i["periods"]}
        datalist.append(d)
    
    dock_slots_df = pd.DataFrame(datalist)
    
    # Changing the period id of all the periods:
    for i in dock_slots_df.itertuples():
        new_id = 1
        for period in i.slots:
            period['id'] = new_id
            new_id += 1
    
    # Converting the message data to the right format:
    columns = ["dock", "slot_num", "status"]
    slots_df = pd.DataFrame(columns=columns)
    available_slots_df = pd.DataFrame(columns=columns)

    # For each dock and slot:
    for tuple in dock_slots_df.itertuples():

        # Creates slots_df with all status = 0:
        buffer = []
        for i in range(0, 1440, 15):
            values = [tuple.dock, i, "0"]
            buff_dict = dict(zip(columns, values))
            buffer.append(buff_dict)

        slots_df = slots_df.append(buffer, True)

        if tuple.slots != []:
            # Convert message to dataframe:
            slots = eval(str(tuple.slots))
            periods_buffer_df = pd.DataFrame(list(slots))
            periods_buffer_df["end"] = periods_buffer_df["start"] + periods_buffer_df["value"]

            # Populating available_slots_df with values from periods_buffer_df:
            for index, row in periods_buffer_df.iterrows():
                i = row['start']
                end = row['end']
                while i < end:
                    available_slots_df.loc[len(available_slots_df.index)] = [tuple.dock, i, "1"]
                    i += 15
                    
        # Updating slots_df with status from available_slots_df:
        slots_df = slots_df.merge(available_slots_df, on=['dock', 'slot_num'], how='left')
        slots_df['status'] = slots_df['status_y']
        slots_df['status'].fillna(slots_df['status_x'], inplace=True)
        slots_df.drop(['status_x', 'status_y'], axis=1, inplace=True)
        slots_df["dock"] = slots_df["dock"].replace("dock_", "", regex=True)

    # Converting slot_num to the expected slot format:
    slots_df['slot'] = slots_df.apply(
        lambda x: (f'{int(x.slot_num / 60):02d}' + ":00") if x.slot_num % 60 == 0 else (f'{int(x.slot_num/60):02d}' + ":" + f'{x.slot_num % 60:02d}'),
        axis=1)

    # Dropping slot_num column:
    slots_df.drop(['slot_num'], axis=1, inplace=True)

    # Reoganizing columns:
    slots_df = slots_df[['dock','slot', 'status']]

    # Writing slots_df to dataiku dataset:
    slots_dataset = dataiku.Dataset(slots_dataset)
    slots_dataset.write_with_schema(slots_df)
    
    # Writing available_slots_df to dataiku dataset:
    available_slots = dataiku.Dataset(cache_dataset)
    available_slots.write_with_schema(dock_slots_df)

    # Return a json status to notify the front if it is ok or not
    return {'status': 'ok', 'message': 'Opened slots are saved!'}


def dataset_to_interface(cache_dataset,compatibility_dataset):
    # Load compatibility:
    compatibility_ds = dataiku.Dataset(str(compatibility_dataset))
    compatibility_df = compatibility_ds.get_dataframe()
    
    # Load periods dataset
    periods_ds = dataiku.Dataset(str(cache_dataset))
    periods_df = periods_ds.get_dataframe()
    
    
    # Create dock_id from dock:
    periods_df["dock_id"] = periods_df["dock"].astype(str).str.slice(5,6).astype('int')
    
    # Merge compatibility and periods:
    periods_df = pd.merge(periods_df, compatibility_df, left_on=['dock_id'], right_on=['dock'], how='left')
    
    # Dropping, reorganizing and renaming columns:
    column_names_to_keep = ['dock_x', 'dock_id', 'slots','compatibility']
    periods_df = periods_df[column_names_to_keep]
    periods_df = periods_df.dropna()

    periods_df = periods_df.rename(columns={"dock_id":"id","dock_x": "name","slots":"periods"})
    periods_df = periods_df[['id', 'name', 'compatibility','periods']]
    
    # Replacing quotes in periods
    periods_df["periods"] = periods_df["periods"].str.replace('\'','"',regex=True)
    

    # Converting to json:
    result = periods_df.to_json(orient="records")
    
    return result